/*-------------------------------------------------*\
|  Amp-clock With RTC chip.                         |
|                                                   |
| CC license for this code (not used libraries)     |
| Attribution-ShareAlike 3.0 Unported (CC BY-SA 3.0)|
| Author: Msquare (Michael Möller), Labitat, 2014   |
\*-------------------------------------------------*/

// Include the full DS1307 library. This also defines and initialises the RTC object
#include <Wire.h>
#include <DS1307.h>

const int METER = 9 ; // output pin for driving meter
const int BUTN1 = 8 ; //   sensing button

// Array which calibrates PWM value where needle is spot on the number
const int CAL[] = { 0, 25, 49, 74, 97, 121, 143, 166, 188, 211, 234, 241 } ;

unsigned long Timer ;      // Utility
boolean SetClock = false ; // Do Clocksetup, otherwise display
int LastMinute = -1 ;      // The last minute showed on needle

void setup() {
  pinMode(METER, OUTPUT);
  analogWrite(METER,0) ;
  pinMode(BUTN1,INPUT) ;
  digitalWrite(BUTN1,HIGH) ; // Pullup resistor 
  analogWrite(METER,127); delay(500); // show activity

  //RTC.startClock();           should be on already
  RTC.ctrl = 0x00; RTC.setCTRL();  // Disable the SqW pulse
  Serial.begin(9600) ;
}

void loop() {
  if ( SetClock ) setuploop() ; else mainloop() ;
}

void SetNeedle( int N, int Unit ) {
/* Set needle to the value, scaled down by Unit, so it falls in 0.0 to 1.0 of the meter scale
  If called with the same arguments, it does nothing (to avoid jiggling the PWM) */
  static int LastN = -1 ; static int LastU = -1 ;
  int CalPoint, AdjVal ;

  // do non-linear scaling from above calibration array - ooohh ugly ugly (should be simplified)
  if ( N != LastN || Unit != LastU ) {
    CalPoint = constrain(N*10/Unit,0,10) ;
    AdjVal = map( N, map(CalPoint, 0,10, 0,Unit), map(CalPoint+1, 0,10, 0,Unit),
                     CAL[CalPoint], CAL[CalPoint+1] ) ;
    analogWrite(METER, constrain(AdjVal,0,242)) ;
    Serial.println(AdjVal) ;
    LastN = N ; LastU = Unit ;
  }
}

void setuploop() {
/* This is a small state machine, ensuring we can enter current HH:MI with one button.
  The needle is waved a bit, then wait for number button pushes that sets a single digit. */

  static byte Entry = 0 ; // State machine
  static byte N ; static byte H = 0 ; static byte T = 0 ; static byte M = 0 ;

  switch (Entry) {
    case 0: case 2: case 4: case 6: wiggle() ; Entry++ ; break ;
    case 1: if (ButtonUpd(&N)) Entry++ ; if (N>2) N=0 ; break ;
    case 3: if (ButtonUpd(&H)) Entry++ ; if (H>9 || (N==2&&H>3)) H=0 ; break ;
    case 5: if (ButtonUpd(&T)) Entry++ ; if (T>5) T=0 ; break ;
    case 7: if (ButtonUpd(&M)) Entry++ ; if (M>9) M=0 ; break ;
    case 8: SetClock = false ;
            //Store a new value in the RTC chip. Date is ignored (random)
            RTC.stopClock();
            //RTC.fillByYMD( year, month, day);
            RTC.fillByHMS(N*10+H,T*10+M,0);
            RTC.setTime();
            RTC.startClock();
            // SetNeedle( MinuteCount, 1440 ) ;
            // Timer = millis() ;
            break ;
  }
}

boolean ButtonUpd(byte *Pdig) {
/* return true when no button change for several seconds. Increment argument for every button push
  put needle at value for feedback. (Wrap is handled by calling function) */

  static byte PrvBtn = HIGH ;
  byte Button ;

  SetNeedle( *Pdig, 10 ) ;
  if ( millis() - Timer > 5 && (Button=digitalRead(BUTN1)) != PrvBtn) {
    // button change, increment digit if push
    if (Button==LOW) (*Pdig)++ ;
    Timer = millis() ; PrvBtn = Button ;
  }
  return millis() - Timer >5000L ;
}

void wiggle() {
/* visual indication that we are changing to next digit. Needle wavers a bit between 4 and 6
  BLOCKING function (cause there is no need not to) */
  SetNeedle( 4, 10 ) ; delay(1300) ;
  SetNeedle( 6, 10 ) ; delay(800) ;
  SetNeedle( 4, 10 ) ; delay(800) ;
  SetNeedle( 6, 10 ) ; delay(1000) ;
  Timer = millis() ; // reset Timer so the wiggle does not count in button timers
}


void mainloop() {
/* Get RTC value, and display the fraction of the day so far.
  If button is pushed, enter a BLOCKING loop that displays 0-11 for 12 hour value (Am/Pm is ignored)
  followed by tens minute and unit minute. Each digit shown for 1.6 seconds
  If button is held down long interval, enter set clock routine */


  if ( millis()-Timer>10000UL) {
    RTC.getTime() ; // Only bother the clock chip once a decasecond
    Timer = millis() ;
  }

  SetNeedle( RTC.hour*60+RTC.minute, 1440 ) ;

  if ( digitalRead(BUTN1)==LOW) {
    // button is pushed. No debounce needed as routine lasts almost 5 seconds
    boolean BtnDwn = true ;
    SetNeedle( RTC.hour%12, 10 ) ;
    for (int b=0; b<=16; b++) {
      delay(100) ;
      BtnDwn &= (digitalRead(BUTN1)==LOW) ;
    }
    if (BtnDwn) {
      SetClock = true ; // change mode, and skip rest of display
    } else { // continue display
      SetNeedle( RTC.minute/10, 10) ;
      delay(1600) ;
      SetNeedle( RTC.minute%10, 10 ) ;
      delay(1800) ;
    }
  }
}


